local addoninfo, mapcoords = ...
local printf = Library.printf.printf
local cron = Library.LibCron
local sprintf = Library.printf.sprintfh

function mapcoords.sign(cur, prev, duration)
  if not cur or not prev or mapcoords.sv.nosign then
    return ""
  end
  local mag = (cur - prev)
  if duration and duration > 0 then
    mag = mag / duration
  end
  if mag < 1.5 and mag > -1.5 then
    return ""
  end
  if cur > prev then
    return "+"
  end
  if cur < prev then
    return "-"
  end
  return ""
end

local atan2 = math.atan2
local pi = math.pi
local floor = math.floor

mapcoords.dirs = {
  "N", "NNW", "NW", "WNW", "W", "WSW", "SW", "SSW", "S", "SSW", "SE", "ESE", "E", "ENE", "NE", "NNE", "N"
}
function mapcoords.dir(dx, dz)
  if dx ~= 0 or dz ~= 0 then
    local angle = floor(((atan2(dx, dz) / pi) + 1.188) * 8)
    return mapcoords.dirs[angle] or angle
  else
    return "--"
  end
end

function mapcoords.update()
  local x = Inspect.Unit.Detail('player')
  local now = Inspect.Time.Frame()
  local elapsed = now - (mapcoords.prev_frame or now)
  if x and x.coordX and x.coordY and x.coordZ then
    local dx, dy, dz, dir = "", "", "", ""
    if mapcoords.prev and mapcoords.prev.coordX and mapcoords.prev.coordY and mapcoords.prev.coordZ then
      dx = mapcoords.sign(x.coordX, mapcoords.prev.coordX, elapsed)
      dy = mapcoords.sign(x.coordY, mapcoords.prev.coordY, elapsed)
      dz = mapcoords.sign(x.coordZ, mapcoords.prev.coordZ, elapsed)
      dir = mapcoords.dir(x.coordX - mapcoords.prev.coordX, x.coordZ - mapcoords.prev.coordZ)
    end
    mapcoords.text:SetText(sprintf("%d%s, %d%s [%d%s] %s",
        x.coordX, dx,
        x.coordZ, dz,
        x.coordY, dy,
        dir))
  end
  mapcoords.prev = x
  mapcoords.prev_frame = now
end

function mapcoords.variables_loaded(event, addon)
  if addon ~= "MiniMapCoords" then
    return
  end
  MiniMapCoords_sv = MiniMapCoords_sv or {}
  printf("/mmc to toggle, -b to move coords, -s to toggle sign, -f # to change font size.")
  mapcoords.sv = MiniMapCoords_sv
  if not mapcoords.ctx then
    mapcoords.ctx = UI.CreateContext("mapcoords")
  end
  mapcoords.ctx:SetStrata("notify")

  if not mapcoords.text then
    mapcoords.text = UI.CreateFrame("Text", "mapcoords_text", mapcoords.ctx)
  end

  mapcoords.text:SetText("")
  mapcoords.place()
end

function mapcoords.place()
  local size = mapcoords.sv.fontsize or 14
  mapcoords.text:SetFontSize(size)
  if mapcoords.sv.below then
    mapcoords.text:SetPoint("CENTER", UI.Native.MapMini, "BOTTOMCENTER", 15, (size - 5))
  else
    mapcoords.text:SetPoint("CENTER", UI.Native.MapMini, "BOTTOMCENTER", 15, -(size + 5))
  end
  mapcoords.text:SetEffectGlow({strength = 10, blurX = 2, blurY = 2})
  if mapcoords.sv.disabled then
    mapcoords.text:SetVisible(false)
    if mapcoords.crontask then
      cron.delete(mapcoords.crontask)
      mapcoords.crontask = nil
    end
  else
    mapcoords.update()
    mapcoords.text:SetVisible(true)
    mapcoords.crontask = cron.new('MiniMapCoords', .1, true, false, mapcoords.update)
  end
end

function mapcoords.slashcommand(args)
  local did_anything = false
  if args.f then
    did_anything = true
    if args.f > 6 and args.f < 30 then
      mapcoords.sv.fontsize = args.f
      mapcoords.sv.disabled = false
      mapcoords.place()
    else
      printf("Font size should be between 6 and 30.")
    end
  end
  if args.b then
    did_anything = true
    mapcoords.sv.disabled = false
    mapcoords.sv.below = not mapcoords.sv.below
    mapcoords.place()
  end
  if args.s then
    did_anything = true
    mapcoords.sv.nosign = not mapcoords.sv.nosign
    mapcoords.update()
  end
  if not did_anything then
    mapcoords.sv.disabled = not mapcoords.sv.disabled
    mapcoords.place()
  end
end

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, mapcoords.variables_loaded, "variable loaded hook")
Library.LibGetOpt.makeslash("bf#s", "MiniMapCoords", "mmc", mapcoords.slashcommand)
